import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:shared_package_flutter/components/showDialog/show_dialog.dart';
import 'package:shared_package_flutter/components/snackbar/snack_bar.dart';
import 'package:shared_package_flutter/components/string_class/string_class.dart';
import 'dart:async';
import 'package:shared_package_flutter/generateCode/generate_code.dart';

class NewUserCodeVerificationParam{
  Color colorApp;
  Function( String ) onResendCode;
  Future<bool> Function( ) onCreate;

  NewUserCodeVerificationParam({
    this.colorApp,
    this.onResendCode,
    this.onCreate
  });
}

class NewUserCodeVerification extends StatefulWidget {
  @override
  _NewUserCodeVerification createState() => _NewUserCodeVerification();
}

class _NewUserCodeVerification extends State<NewUserCodeVerification> with SingleTickerProviderStateMixin {
  final GlobalKey<FormState> _keyValidationForm = GlobalKey<FormState>();
  NewUserCodeVerificationParam _paramModel;

  String codeVerification;

  TextEditingController _textEditingControllerFirstDigit = TextEditingController( );
  TextEditingController _textEditingControllerSecondDigit = TextEditingController( );
  TextEditingController _textEditingControllerThirdDigit = TextEditingController( );
  TextEditingController _textEditingControllerFourthDigit = TextEditingController( );
  TextEditingController _textEditingControllerFifithDigit = TextEditingController( );

  FocusNode _focusFirstDigit = FocusNode();
  FocusNode _focusSecondDigit = FocusNode();
  FocusNode _focusThirdDigit = FocusNode();
  FocusNode _focusFourthDigit = FocusNode();
  FocusNode _focusFifithDigit = FocusNode();

  // Timeout de 30 segundos para reenviar o email
  final int timerMaxSeconds = 30;
  int currentSeconds = 0;
  Timer timerCodeVerification;

  int _stateSave = 0;
  bool showResendCodeVerification = false;

  @override
  void initState() {
    super.initState();

    startTimeout();
  }

  @override
  void dispose() {
    // this State gets disposed.
    super.dispose();

    currentSeconds = 0;
    timerCodeVerification.cancel();
  }

  startTimeout() {
    timerCodeVerification = Timer.periodic( Duration( seconds: 1 ), ( timer ) {
      setState( () {
        currentSeconds = timer.tick;

        if ( timer.tick >= timerMaxSeconds ){
          currentSeconds = 0;
          timer.cancel();

          setState(() {
            showResendCodeVerification = true;
          });
        }
      });
    });
  }

  // https://stackoverflow.com/questions/54610121/flutter-countdown-timer
  String get timerText => "0:${(timerMaxSeconds - currentSeconds).toString().padLeft(2, "0")}";

  Widget buttonSave(){
    if( _stateSave == 0 ){
      return Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Icon( Icons.save ),
          Padding( padding: EdgeInsets.only( right: 15 ) ),
          Text(
            "Validar",
            style: TextStyle( fontSize: 20.0 ),
          )
        ],
      );
    }
    else if( _stateSave == 1 ){
      return SizedBox(
        height: 20,
        width: 20,
        child: CircularProgressIndicator(
          value: null,
          valueColor: AlwaysStoppedAnimation<Color>( Colors.white ),
        ),
      );
    }
    else if( _stateSave == 2 ){
      return Icon( Icons.check, color: Colors.white );
    }
    else{
      return Text(
        "Erro",
        style: TextStyle( fontSize: 20.0 ),
      );
    }
  }

  void buttonSaveUser() async{

    if( _stateSave == 3 ){
      setState(() {
        _stateSave = 0;
      });
    }

    try{

      if ( !StringClass.isNullOrEmpty( codeVerification ) ) {

        String codeVerification = "${_textEditingControllerFirstDigit.text}"
        + " ${_textEditingControllerSecondDigit.text}"
        + " ${_textEditingControllerThirdDigit.text}"
        + " ${_textEditingControllerFourthDigit.text}"
        + " ${_textEditingControllerFifithDigit.text}";

        if( codeVerification.compareTo( codeVerification ) == 0 ){

          if( !await _paramModel.onCreate( ) ){

            setState(() {
              _stateSave = 3;
            });

            ShowSnackBar.show(
              context,
              SnackBarParam(
                "Erro ao criar usuário"
              )
            );
          }
        }
        else{

          setState(() {
            _stateSave = 3;
          });

          ShowDialog.show(
            context,
            ShowDialogParam(
              "Erro",
              "Código de validação está incorreto"
            )
          );
        }
      }
      else{

        setState(() {
          _stateSave = 3;
        });

        ShowDialog.show(
          context,
          ShowDialogParam(
            "Erro",
            "Ocorreu um erro ao tentar verificar o código"
          )
        );
      }

    }catch( exception ){
      setState(() {
        _stateSave = 3;
      });

      ShowDialog.show(
        context,
        ShowDialogParam(
          "Erro",
          "Ocorreu um erro ao criar o usuário"
        )
      );
    }
  }

  bool allDigitsComplete(){

    return !StringClass.isNullOrEmpty( _textEditingControllerFirstDigit.text )
      && !StringClass.isNullOrEmpty( _textEditingControllerSecondDigit.text )
      && !StringClass.isNullOrEmpty( _textEditingControllerThirdDigit.text )
      && !StringClass.isNullOrEmpty( _textEditingControllerFourthDigit.text )
      && !StringClass.isNullOrEmpty( _textEditingControllerFifithDigit.text );
  }

  @override
  Widget build( BuildContext context ){

    Object obj = ModalRoute.of( context ).settings.arguments;
    if( null != obj && obj.runtimeType == NewUserCodeVerificationParam ){
      _paramModel = obj as NewUserCodeVerificationParam;
    }
    else{

      return Scaffold(
        backgroundColor: Colors.grey,
        body: Center(
          child: Text(
            "Erro ao abrir a tela para criar usuário, parametros não informado",
            style: TextStyle( fontSize: 20 )
          )
        )
      );
    }

    return Scaffold(
      appBar: AppBar(
        title: Text( "Criar usuário" ),
      ),
      body: Container(
        padding: EdgeInsets.only( left: 20.0, right: 20.0 ),
        child: Form(
          key: _keyValidationForm,
          child: ListView(
            children: <Widget>[
              Container(
                padding: EdgeInsets.only( top: 35 ),
                child: Center(
                  child: Text(
                    "Foi enviado o código para o email.",
                    style: TextStyle( fontSize: 20 ),
                  )
                )
              ),
              (
                showResendCodeVerification
                ? Container(
                  padding: EdgeInsets.only( top: 35 ),
                  child: Center(
                    child: InkWell(
                      child: Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: <Widget>[
                          Icon( Icons.sync ),
                          Padding( padding: EdgeInsets.only( left: 5 ) ),
                          Text(
                            "Reenviar código",
                            style: TextStyle( fontSize: 18, color: Colors.blue ),
                          )
                        ],
                      ),
                      onTap: () async {
                        setState(() {
                          showResendCodeVerification = false;
                        });

                        codeVerification = GenerateCode.code5Digits();

                        _paramModel.onResendCode( codeVerification );

                        startTimeout();
                      },
                    )
                  )
                )
                : Container(
                  padding: EdgeInsets.only( top: 35 ),
                  child: Center(
                    child: Text(
                      "Para reenviar o código, espere: $timerText",
                      style: TextStyle( fontSize: 18 )
                      //textScaleFactor: 1 + controller.value,
                    )
                  )
                )
              ),
              Container(
                padding: EdgeInsets.only( top: 45 ),
                child: Center(
                  child: Text(
                    "Código",
                    style: TextStyle( fontSize: 26 ),
                  )
                )
              ),
              Container(
                padding: EdgeInsets.only( top: 15 ),
                child: Row(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    Container(
                      width: 50,
                      child: TextFormField(
                        controller: _textEditingControllerFirstDigit,
                        keyboardType: TextInputType.number,
                        textInputAction: TextInputAction.next,
                        textAlign: TextAlign.center,
                        style: TextStyle( fontSize: 30 ),
                        inputFormatters: [
                          WhitelistingTextInputFormatter.digitsOnly,
                          new LengthLimitingTextInputFormatter(1)
                        ],
                        onChanged: (value){
                          FocusScope.of( context ).requestFocus( _focusSecondDigit );
                        },
                        focusNode: _focusFirstDigit,
                        autofocus: true,
                      )
                    ),
                    Padding( padding: EdgeInsets.only( left: 20 ) ),
                    Container(
                      width: 50,
                      child: RawKeyboardListener(
                        onKey: (RawKeyEvent event){
                          // 4295426090 voltar do teclado
                          if(event.runtimeType == RawKeyDownEvent && event.logicalKey.keyId == 4295426090){
                            FocusScope.of( context ).requestFocus( _focusFirstDigit );
                          }
                          //Tira o foco para fechar o teclado
                          else{
                            if( allDigitsComplete() ){
                              FocusScope.of( context ).requestFocus( new FocusNode() );
                            }
                          }
                        },
                        focusNode: FocusNode(),
                        child: TextFormField(
                          controller: _textEditingControllerSecondDigit,
                          keyboardType: TextInputType.number,
                          textInputAction: TextInputAction.next,
                          textAlign: TextAlign.center,
                          style: TextStyle( fontSize: 30 ),
                          inputFormatters: [
                            WhitelistingTextInputFormatter.digitsOnly,
                            new LengthLimitingTextInputFormatter(1)
                          ],
                          onChanged: (value){
                            FocusScope.of( context ).requestFocus( _focusThirdDigit );
                          },
                          focusNode: _focusSecondDigit
                        )
                      )
                    ),
                    Padding( padding: EdgeInsets.only( left: 20 ) ),
                    Container(
                      width: 50,
                      child: RawKeyboardListener(
                        onKey: (RawKeyEvent event){
                          // 4295426090 voltar do teclado
                          if(event.runtimeType == RawKeyDownEvent && event.logicalKey.keyId == 4295426090){
                            FocusScope.of( context ).requestFocus( _focusSecondDigit );
                          }
                          // Tira o foco para fechar o teclado
                          else{
                            if( allDigitsComplete() ){
                              FocusScope.of( context ).requestFocus( new FocusNode() );
                            }
                          }
                        },
                        focusNode: FocusNode(),
                        child: TextFormField(
                          controller: _textEditingControllerThirdDigit,
                          keyboardType: TextInputType.number,
                          textInputAction: TextInputAction.next,
                          textAlign: TextAlign.center,
                          style: TextStyle( fontSize: 30 ),
                          inputFormatters: [
                            WhitelistingTextInputFormatter.digitsOnly,
                            new LengthLimitingTextInputFormatter(1)
                          ],
                          onChanged: (value){
                            FocusScope.of( context ).requestFocus( _focusFourthDigit );
                          },
                          focusNode: _focusThirdDigit
                        )
                      )
                    ),
                    Padding( padding: EdgeInsets.only( left: 20 ) ),
                    Container(
                      width: 50,
                      child: RawKeyboardListener(
                        onKey: (RawKeyEvent event){
                          // 4295426090 voltar do teclado
                          if(event.runtimeType == RawKeyDownEvent && event.logicalKey.keyId == 4295426090){
                            FocusScope.of( context ).requestFocus( _focusThirdDigit );
                          }
                          // Tira o foco para fechar o teclado
                          else{
                            if( allDigitsComplete() ){
                              FocusScope.of( context ).requestFocus( new FocusNode() );
                            }
                          }
                        },
                        focusNode: FocusNode(),
                        child: TextFormField(
                          controller: _textEditingControllerFourthDigit,
                          keyboardType: TextInputType.number,
                          textInputAction: TextInputAction.next,
                          textAlign: TextAlign.center,
                          style: TextStyle( fontSize: 30 ),
                          inputFormatters: [
                            WhitelistingTextInputFormatter.digitsOnly,
                            new LengthLimitingTextInputFormatter(1)
                          ],
                          onChanged: (value){
                            FocusScope.of( context ).requestFocus( _focusFifithDigit );
                          },
                          focusNode: _focusFourthDigit
                        )
                      )
                    ),
                    Padding( padding: EdgeInsets.only( left: 20 ) ),
                    Container(
                      width: 50,
                      child: RawKeyboardListener(
                        onKey: (RawKeyEvent event){
                          // 4295426090 voltar do teclado
                          if(event.runtimeType == RawKeyDownEvent && event.logicalKey.keyId == 4295426090){
                            FocusScope.of( context ).requestFocus( _focusFourthDigit );
                          }
                          // Tira o foco para fechar o teclado
                          else{
                            if( allDigitsComplete() ){
                              FocusScope.of( context ).requestFocus( new FocusNode() );
                            }
                          }
                        },
                        focusNode: FocusNode(),
                        child: TextFormField(
                          controller: _textEditingControllerFifithDigit,
                          keyboardType: TextInputType.number,
                          textInputAction: TextInputAction.next,
                          textAlign: TextAlign.center,
                          style: TextStyle( fontSize: 30 ),
                          inputFormatters: [
                            WhitelistingTextInputFormatter.digitsOnly,
                            new LengthLimitingTextInputFormatter(1)
                          ],
                          focusNode: _focusFifithDigit
                        )
                      )
                    )
                  ],
                )
              ),
              Container(
                margin: EdgeInsets.only( top: 50.0, left: 80.0, right: 80.0 ),
                width: double.infinity,
                child: RaisedButton(
                  color: _stateSave == 3 ? Colors.redAccent : _paramModel.colorApp,
                  textColor: Colors.white,
                  elevation: 5.0,
                  padding: EdgeInsets.only( top: 16.0, bottom: 16.0 ),
                  child: buttonSave(),
                  onHighlightChanged: ( isPressed ){
                    if( _keyValidationForm.currentState.validate() ){
                      if( _stateSave == 0 ){
                        setState(() {
                          _stateSave = 1;
                        });
                      }
                    }
                  },
                  onPressed: buttonSaveUser
                )
              )
            ],
          )
        )
      )
    );
  }
}