import 'package:flutter/material.dart';
import 'package:shared_package_flutter/components/showDialog/show_dialog.dart';
import 'package:shared_package_flutter/regularExpression/regular_expression.dart';

class UserCreateParam{
  Color colorApp;
  Future<bool> Function( String ) onCreate;

  UserCreateParam({
    this.colorApp,
    this.onCreate
  });
}

class UserCreate extends StatefulWidget{
  @override
  _UserCreate createState() => _UserCreate();
}

class _UserCreate extends State<UserCreate>{
  final GlobalKey<FormState> _keyValidationForm = GlobalKey<FormState>();
  UserCreateParam _paramModel;

  TextEditingController _textEditingControllerName = TextEditingController( );
  TextEditingController _textEditingControllerEmail = TextEditingController( );
  TextEditingController _textEditingControllerPassword = TextEditingController( );
  TextEditingController _textEditingControllerRepeatPassword = TextEditingController( );

  final FocusNode _emailFocus = FocusNode();
  final FocusNode _passwordFocus = FocusNode();
  final FocusNode _repeatPasswordFocus = FocusNode();
  final FocusNode _buttonSaveFocus = FocusNode();

  int _stateSave = 0;

  Widget buttonSave(){
    if( _stateSave == 0 ){
      return Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Icon( Icons.save ),
          Padding( padding: EdgeInsets.only( right: 15 ) ),
          Text(
            "Salvar",
            style: TextStyle( fontSize: 20.0 ),
          )
        ],
      );
    }
    else if( _stateSave == 1 ){
      return SizedBox(
        height: 20,
        width: 20,
        child: CircularProgressIndicator(
          value: null,
          valueColor: AlwaysStoppedAnimation<Color>( Colors.white ),
        ),
      );
    }
    else if( _stateSave == 2 ){
      return Icon( Icons.check, color: Colors.white );
    }
    else{
      return Text(
        "Erro",
        style: TextStyle( fontSize: 20.0 ),
      );
    }
  }

  void buttonSaveUser() async{

    if( _stateSave == 3 ){
      setState(() {
        _stateSave = 0;
      });
    }

    try{

      if ( _keyValidationForm.currentState.validate() ) {

        if( _textEditingControllerPassword.text.toLowerCase() == _textEditingControllerRepeatPassword.text.toLowerCase() ){

          if( !await _paramModel.onCreate( _textEditingControllerEmail.text ) ){

            setState(() {
              _stateSave = 3;
              FocusScope.of( context ).requestFocus( _buttonSaveFocus );
            });

            ShowDialog.show(
              context,
              ShowDialogParam(
                "Erro",
                "Ocorreu um erro ao tentar criar a conta"
              )
            );
          }
        }
        else{

          setState(() {
            _stateSave = 3;
            FocusScope.of( context ).requestFocus( _buttonSaveFocus );
          });

          ShowDialog.show(
              context,
              ShowDialogParam(
                "Erro",
                "A senha de confirmação está diferente"
              )
            );
        }
      }

    }catch( exception ){
      setState(() {
        _stateSave = 3;
        FocusScope.of( context ).requestFocus( _buttonSaveFocus );
      });

      ShowDialog.show(
        context,
        ShowDialogParam(
          "Erro",
          "Ocorreu um erro ao criar o usuário"
        )
      );
    }
  }

  @override
  Widget build( BuildContext context ){

    Object obj = ModalRoute.of( context ).settings.arguments;
    if( null != obj && obj.runtimeType == UserCreateParam ){
      _paramModel = obj as UserCreateParam;
    }
    else{

      return Scaffold(
        backgroundColor: Colors.grey,
        body: Center(
          child: Text(
            "Erro ao abrir a tela para criar usuário, parametros não informado",
            style: TextStyle( fontSize: 20 )
          )
        )
      );
    }

    return Scaffold(
      appBar: AppBar(
        title: Text( "Criar usuário" ),
      ),
      body: Container(
        padding: EdgeInsets.only( left: 20.0, right: 20.0 ),
        child: Form(
          key: _keyValidationForm,
          child: ListView(
            children: <Widget>[
              Container(
                padding: EdgeInsets.only( top: 20 ),
                child: TextFormField(
                  controller: _textEditingControllerName,
                  keyboardType: TextInputType.text,
                  textInputAction: TextInputAction.next,
                  validator: _validateName,
                  autofocus: true,
                  onFieldSubmitted: ( String value ) {
                    FocusScope.of( context ).requestFocus( _emailFocus );
                  },
                  decoration: InputDecoration(
                    labelText: "Nome"
                  ),
                )
              ),
              Container(
                padding: EdgeInsets.only( top: 20 ),
                child: TextFormField(
                  focusNode: _emailFocus,
                  controller: _textEditingControllerEmail,
                  keyboardType: TextInputType.emailAddress,
                  textInputAction: TextInputAction.next,
                  validator: _validateEmail,
                  onFieldSubmitted: ( String value ) {
                    FocusScope.of( context ).requestFocus( _passwordFocus );
                  },
                  decoration: InputDecoration(
                    labelText: "Email"
                  ),
                )
              ),
              Container(
                padding: EdgeInsets.only( top: 20 ),
                child: TextFormField(
                  focusNode: _passwordFocus,
                  controller: _textEditingControllerPassword,
                  keyboardType: TextInputType.text,
                  textInputAction: TextInputAction.next,
                  validator: _validatePassword,
                  obscureText: true,
                  onFieldSubmitted: ( String value ) {
                    FocusScope.of( context ).requestFocus( _repeatPasswordFocus );
                  },
                  decoration: InputDecoration(
                    labelText: "Senha"
                  ),
                )
              ),
              Container(
                padding: EdgeInsets.only( top: 20 ),
                child: TextFormField(
                  focusNode: _repeatPasswordFocus,
                  controller: _textEditingControllerRepeatPassword,
                  keyboardType: TextInputType.text,
                  textInputAction: TextInputAction.next,
                  validator: _validateRepeatPassword,
                  obscureText: true,
                  onFieldSubmitted: ( String value ) {
                    FocusScope.of( context ).requestFocus( _buttonSaveFocus );
                  },
                  decoration: InputDecoration(
                    labelText: "Confirme a senha"
                  ),
                )
              ),
              Container(
                margin: EdgeInsets.only( top: 50.0, left: 80.0, right: 80.0 ),
                width: double.infinity,
                child: RaisedButton(
                  focusNode: _buttonSaveFocus,
                  color: _stateSave == 3 ? Colors.redAccent : _paramModel.colorApp,
                  textColor: Colors.white,
                  elevation: 5.0,
                  padding: EdgeInsets.only( top: 16.0, bottom: 16.0 ),
                  child: buttonSave(),
                  onHighlightChanged: ( isPressed ){
                    if( _keyValidationForm.currentState.validate() ){
                      if( _stateSave == 0 ){
                        setState(() {
                          _stateSave = 1;
                        });
                      }
                    }
                  },
                  onPressed: buttonSaveUser
                )
              )
            ],
          )
        )
      )
    );
  }
}

String _validateName( String value ) {
  return value.trim().isEmpty ? "Preencha o campo nome" : null;
}

String _validateEmail( String value ) {
  if ( !RegularExpression.email( ).hasMatch( value ) )
    return "Preencha o campo email";
  else
    return null;
}

String _validatePassword( String value ) {
  return value.trim().isEmpty ? "Preencha o campo senha" : null;
}

String _validateRepeatPassword( String value ) {
  return value.trim().isEmpty ? "Preencha o campo confirme a senha" : null;
}