import 'dart:async';
import 'dart:io';
import 'package:mailer/mailer.dart';
import 'package:mailer/smtp_server/gmail.dart';
import 'package:shared_package_flutter/components/string_class/string_class.dart';

class MailParam{
  String titleEmail;
  String emailDestination;
  String subject;
  String htmlText;
  File fileAttachment;

  MailParam(
    this.titleEmail,
    this.emailDestination,
    this.subject,
    this.htmlText,
    {
      this.fileAttachment
    }
  );
}

class Mail {
  final String email;
  final String password;

  Mail( this.email, this.password );

  Future<bool> sendMail( MailParam _param ) async {
    bool success = false;

    if( null != _param ){
      if( !StringClass.isNullOrEmpty( email ) ){

        // https://pub.dev/packages/mailer
        final smtpServer = gmail( this.email, this.password); 
        
        // Create our message.
        final message = Message()
          ..from = Address( "noreply@gmail.com", _param.titleEmail )
          ..recipients.add( _param.emailDestination )
          ..subject = _param.subject
          ..attachments.add( FileAttachment( _param.fileAttachment ) )
          ..html = _param.htmlText;

        try {

          final sendReport = await send( message, smtpServer );
          print( "Email enviado: ${sendReport.toString()}" );

          success = true;

        } on MailerException catch (e) {
          print( "Erro ao enviar email" );
          for (var p in e.problems) {
            print( "Erro: ${p.code}: ${p.msg}" );
          }
        }
      }
      else{
        print( "Não foi passado nenhum email" );
      }
    }
    else{
      print( "Parametro do email não definido" );
    }

    return success;
  }

  /*Future<bool> sendCodeVerification( String email, String codeVerification ) async {
    bool success = false;

    String data = await getFileData( "assets/pedidos.txt" );

    await writeFile( data );
    final file = await _localFile;
    //String data2 = await file.readAsString();

    /*//final file = new File( "assets/pedidos.txt" );
    final path = await _localPath;
    final file = new File( "$path/pedidos.txt" );
    String data2 = await file.readAsString();

    Stream<List<int>> inputStream = file.openRead();
    Attachment attachment;*/

    if( !StringClass.isNullOrEmpty( email ) ){

      // https://pub.dev/packages/mailer
      final smtpServer = gmail( this.email, this.password ); 
      
      // Create our message.
      final message = Message()
        ..from = Address( "noreply@gmail.com", "App" )
        ..recipients.add( email )
        //..ccRecipients.addAll(['destCc1@example.com', 'destCc2@example.com'])
        //..bccRecipients.add(Address('bccAddress@example.com'))
        ..subject = "$codeVerification é o seu código de acesso"
        //..attachments.add( FileAttachment( File( "assets/pedidos.txt" ), fileName: "pedidos.txt" ) )
        ..attachments.add(
          FileAttachment( file, fileName: "pedidos.txt"/*, contentType: "multipart/mixed"*/ )
          //StreamAttachment( file.openRead(), "multipart/mixed", fileName: "pedidos.txt" )
        )
        //..text = 'This is the plain text.\nThis is line 2 of the text part.';
        ..html = "<h1>Código de acesso</h1><p>Seu código de acesso para o aplicativo:</p><h2>$codeVerification</h2>";

      try {

        final sendReport = await send( message, smtpServer );
        print( "Email enviado: ${sendReport.toString()}" );

        success = true;

      } on MailerException catch (e) {
        print( "Erro ao enviar email" );
        for (var p in e.problems) {
          print( "Erro: ${p.code}: ${p.msg}" );
        }
      }
    }
    else{
      print( "Não foi passado nenhum email" );
    }

    return success;
  }*/
}